@@ -1,17 +1,17 @@
-import audalign.filehandler as filehandler
-import audalign.fingerprint as fingerprint
-import audalign.recognize as recognize
-import audalign.align as align
-import audalign.visrecognize as visrecognize
-import audalign.correcognize as correcognize
-from pydub.exceptions import CouldntDecodeError
-from pydub.utils import mediainfo
-from typing import Tuple
-from functools import partial
+import json
 import multiprocessing
 import os
 import pickle
-import json
+from functools import partial
+
+from pydub.utils import mediainfo
+
+import audalign.align as align
+import audalign.correcognize as correcognize
+import audalign.filehandler as filehandler
+import audalign.fingerprint as fingerprint
+import audalign.recognize as recognize
+import audalign.visrecognize as visrecognize
 
 
 class Audalign:
@@ -244,7 +244,9 @@ class Audalign:
                 name_checker.add(self.file_names[i])
                 i += 1
 
-    def fingerprint_directory(self, path: str, plot=False, extensions=["*"]) -> None:
+    def fingerprint_directory(
+        self, path: str, plot=False, extensions=["*"], _file_audsegs: dict = None
+    ) -> None:
         """
         Fingerprints all files in given directory and all subdirectories
 
@@ -252,13 +254,16 @@ class Audalign:
             path (str): path to directory to be fingerprinted
             plot (boolean): if true, plots the peaks to be fingerprinted on a spectrogram
             extensions (list[str]): specify which extensions to fingerprint
+            _file_audsegs (dict, None): For internal use with fine align
 
         Returns
         -------
         None
         """
 
-        result = self._fingerprint_directory(path, plot, extensions)
+        result = self._fingerprint_directory(
+            path, plot=plot, extensions=extensions, _file_audsegs=_file_audsegs
+        )
 
         if result:
             for processed_file in result:
@@ -270,7 +275,13 @@ class Audalign:
                     self.file_names.append(processed_file[0])
                     self.total_fingerprints += len(processed_file[1])
 
-    def _fingerprint_directory(self, path: str, plot=False, extensions=["*"]):
+    def _fingerprint_directory(
+        self,
+        path: str,
+        plot: bool = False,
+        extensions: list = ["*"],
+        _file_audsegs: dict = None,
+    ):
         """
         Worker function for fingerprint_directory
 
@@ -289,6 +300,8 @@ class Audalign:
             file_names = filehandler.find_files(path, extensions)
         elif type(path) == list:
             file_names = zip(path, ["_"] * len(path))
+        elif path is None and _file_audsegs is not None:
+            file_names = zip(_file_audsegs.keys(), ["_"] * len(_file_audsegs))
 
         filenames_to_fingerprint = []
         for filename, _ in file_names:  # finds all files to fingerprint
@@ -302,8 +315,14 @@ class Audalign:
             print("Directory contains 0 files or could not be found")
             return
 
+        if _file_audsegs is not None:
+            filenames_to_fingerprint = [
+                (filename, _file_audsegs[filename])
+                for filename in filenames_to_fingerprint
+            ]
+
         _fingerprint_worker_directory = partial(
-            _fingerprint_worker,
+            fingerprint._fingerprint_worker,
             hash_style=self.hash_style,
             plot=plot,
             accuracy=self.accuracy,
@@ -400,7 +419,7 @@ class Audalign:
         [file_name, hashes]
         """
 
-        file_name, hashes = _fingerprint_worker(
+        file_name, hashes = fingerprint._fingerprint_worker(
             file_path,
             self.hash_style,
             start_end=start_end,
@@ -418,6 +437,7 @@ class Audalign:
         locality: float = None,
         locality_filter_prop: float = None,
         start_end: tuple = None,
+        max_lags: float = None,
         *args,
         **kwargs,
     ) -> None:
@@ -434,6 +454,7 @@ class Audalign:
             locality (float): filters matches to only count within locality. In seconds
             locality_filter_prop (int, float,optional): within each offset, filters locality tuples by proportion of highest confidence to tuple confidence
             start_end (tuple(float, float), optional): Silences before and after start and end. (0, -1) Silences last second, (5.4, 0) silences first 5.4 seconds
+            max_lags (float, optional): Maximum lags in seconds.
 
         Returns
         -------
@@ -455,6 +476,7 @@ class Audalign:
             locality=locality,
             locality_filter_prop=locality_filter_prop,
             start_end=start_end,
+            max_lags=max_lags,
             *args,
             **kwargs,
         )
@@ -466,8 +488,9 @@ class Audalign:
         start_end_target: tuple = None,
         start_end_against: tuple = None,
         filter_matches: float = 0.5,
-        match_len_filter: int = 30,
+        match_len_filter: int = None,
         sample_rate: int = fingerprint.DEFAULT_FS,
+        max_lags: float = None,
         plot: bool = False,
         **kwargs,
     ):
@@ -485,6 +508,7 @@ class Audalign:
             match_len_filter (int, optional): Limits number of matches returned. Defaults to 30.
             sample_rate (int, optional): Decodes audio file to this sample rate. Defaults to fingerprint.DEFAULT_FS.
             plot (bool, optional): Plots. Defaults to False.
+            max_lags (float, optional): Maximum lags in seconds.
             kwargs: additional arguments for scipy.signal.find_peaks.
 
         Returns:
@@ -499,6 +523,7 @@ class Audalign:
             match_len_filter=match_len_filter,
             sample_rate=sample_rate,
             plot=plot,
+            max_lags=max_lags,
             **kwargs,
         )
 
@@ -565,9 +590,11 @@ class Audalign:
         against_directory: str,
         start_end: tuple = None,
         filter_matches: float = 0.5,
-        match_len_filter: int = 30,
+        match_len_filter: int = None,
         sample_rate: int = fingerprint.DEFAULT_FS,
+        max_lags: float = None,
         plot: bool = False,
+        _file_audsegs: dict = None,
         **kwargs,
     ):
         """Uses cross correlation to find alignment
@@ -583,6 +610,8 @@ class Audalign:
             match_len_filter (int, optional): Limits number of matches returned. Defaults to 30.
             sample_rate (int, optional): Decodes audio file to this sample rate. Defaults to fingerprint.DEFAULT_FS.
             plot (bool, optional): Plots. Defaults to False.
+            max_lags (float, optional): Maximum lags in seconds.
+            _file_audsegs (dict, optional): For use with align.
             kwargs: additional arguments for scipy.signal.find_peaks.
 
         Returns:
@@ -596,6 +625,8 @@ class Audalign:
             match_len_filter=match_len_filter,
             sample_rate=sample_rate,
             plot=plot,
+            max_lags=max_lags,
+            _file_audsegs=_file_audsegs,
             **kwargs,
         )
 
@@ -816,8 +847,6 @@ class Audalign:
                     calc_mse=calc_mse,
                 )
             elif technique == "correlation":
-                if filter_matches is None:
-                    filter_matches = 0.5
                 alignment = self.correcognize_directory(
                     target_file_path=target_file,
                     against_directory=directory_path,
@@ -877,6 +906,7 @@ class Audalign:
             )
 
             files_shifts["match_info"] = total_alignment
+            files_shifts["names_and_paths"] = file_names_and_paths
             return files_shifts
 
         finally:
@@ -896,6 +926,7 @@ class Audalign:
         locality: float = None,
         locality_filter_prop: float = None,
         cor_sample_rate: int = fingerprint.DEFAULT_FS,
+        max_lags: float = None,
         **kwargs,
     ):
         """
@@ -914,12 +945,13 @@ class Audalign:
             locality (float): Only recognizes against fingerprints in given width. In seconds
             locality_filter_prop (int, float,optional): within each offset, filters locality tuples by proportion of highest confidence to tuple confidence
             cor_sample_rate (int): Sampling rate for correlation
+            max_lags (float, optional): Maximum lags in seconds for correlation.
             **kwargs: Additional arguments for finding peaks in correlation
 
         Returns
         -------
             files_shifts (dict{float}): dict of file name with shift as value
-        """  # TODO Docs
+        """
         filename_list = [filename_a, filename_b, *filenames]
         # print(filename_a, filename_b, *filenames, sep="\n")
         # print(filename_list)
@@ -933,6 +965,7 @@ class Audalign:
             locality=locality,
             locality_filter_prop=locality_filter_prop,
             cor_sample_rate=cor_sample_rate,
+            max_lags=max_lags,
             **kwargs,
         )
 
@@ -946,6 +979,7 @@ class Audalign:
         locality: float = None,
         locality_filter_prop: float = None,
         cor_sample_rate: int = fingerprint.DEFAULT_FS,
+        max_lags: float = None,
         **kwargs,
     ):
         """
@@ -976,6 +1010,7 @@ class Audalign:
             locality=locality,
             locality_filter_prop=locality_filter_prop,
             cor_sample_rate=cor_sample_rate,
+            max_lags=max_lags,
             **kwargs,
         )
 
@@ -990,6 +1025,8 @@ class Audalign:
         locality: float = None,
         locality_filter_prop: float = None,
         cor_sample_rate: int = fingerprint.DEFAULT_FS,
+        max_lags: float = None,
+        fine_aud_file_dict: dict = None,
         **kwargs,
     ):
 
@@ -1015,18 +1052,23 @@ class Audalign:
                 if file_dir:
                     self.fingerprint_directory(file_dir)
                 else:
-                    self.fingerprint_directory(filename_list)
+                    self.fingerprint_directory(
+                        filename_list,
+                        _file_audsegs=fine_aud_file_dict,
+                    )
             elif technique == "correlation":
-                if filter_matches is None:
-                    filter_matches = 0.5
                 if file_dir:
                     self.file_names = filehandler.get_audio_files_directory(file_dir)
+                elif fine_aud_file_dict:
+                    self.file_names = [
+                        os.path.basename(x) for x in fine_aud_file_dict.keys()
+                    ]
                 else:
                     self.file_names = [os.path.basename(x) for x in filename_list]
 
             else:
                 raise NameError(
-                    f'Technique parameter must be fingerprint, visual, or correlation, not "{technique}"'
+                    f'Technique parameter must be fingerprints, visual, or correlation, not "{technique}"'
                 )
 
             total_alignment = {}
@@ -1035,6 +1077,11 @@ class Audalign:
             if file_dir:
                 file_list = filehandler.find_files(file_dir)
                 dir_or_list = file_dir
+            elif fine_aud_file_dict:
+                file_list = zip(
+                    fine_aud_file_dict.keys(), ["_"] * len(fine_aud_file_dict)
+                )
+                dir_or_list = fine_aud_file_dict.keys()
             else:
                 file_list = zip(filename_list, ["_"] * len(filename_list))
                 dir_or_list = filename_list
@@ -1048,6 +1095,7 @@ class Audalign:
                             filter_matches=filter_matches,
                             locality=locality,
                             locality_filter_prop=locality_filter_prop,
+                            max_lags=max_lags,
                         )
                     elif technique == "correlation":
                         alignment = self.correcognize_directory(
@@ -1055,6 +1103,8 @@ class Audalign:
                             dir_or_list,
                             filter_matches=filter_matches,
                             sample_rate=cor_sample_rate,
+                            _file_audsegs=fine_aud_file_dict,
+                            max_lags=max_lags,
                             **kwargs,
                         )
                     file_names_and_paths[name] = file_path
@@ -1083,6 +1133,7 @@ class Audalign:
             )
 
             files_shifts["match_info"] = total_alignment
+            files_shifts["names_and_paths"] = file_names_and_paths
             return files_shifts
 
         finally:
@@ -1090,6 +1141,90 @@ class Audalign:
             self.fingerprinted_files = temp_fingerprinted_files
             self.total_fingerprints = temp_total_fingerprints
 
+    def fine_align(
+        self,
+        results,
+        destination_path: str = None,
+        write_extension: str = None,
+        technique="correlation",
+        filter_matches: float = None,
+        max_lags: float = 2,
+        locality: float = None,
+        locality_filter_prop: float = None,
+        cor_sample_rate: int = fingerprint.DEFAULT_FS,
+        match_index: int = 0,
+        strength_stat: str = CONFIDENCE,
+        **kwargs,
+    ):
+        """
+        Finds matches and relative offsets for all files in directory_path, aligns them, and writes them to destination_path
+
+        Args
+            results (dict): results from previous alignments.
+            directory_path (str): String of directory for alignment
+            destination_path (str): String of path to write alignments to
+            write_extension (str): if given, writes all alignments with given extension (ex. ".wav" or "wav")
+            technique (str): either "fingerprints" or "correlation"
+            filter_matches (float): filters based on confidence.
+            locality (float): Only recognizes against fingerprints in given width. In seconds
+            locality_filter_prop (int, float,optional): within each offset, filters locality tuples by proportion of highest confidence to tuple confidence
+            cor_sample_rate (int): Sampling rate for correlation
+            match_index (int): reorders the input results to the given match index.
+            strength_stat (str): strength stat for finding proper alignments. Defaults to confidence.
+            **kwargs: Additional arguments for finding peaks in correlation
+
+        Returns
+        -------
+            files_shifts (dict{float}): dict of file name with shift as value. Includes match_info, fine_match_info, and names_and_paths
+        """
+        print("Fine Aligning...")
+
+        if match_index != 0:
+            recalc_shifts_results = align.recalc_shifts_index(
+                results, strength_stat=strength_stat, match_index=match_index
+            )
+            paths_audio = filehandler.shift_get_files(
+                recalc_shifts_results, sample_rate=cor_sample_rate
+            )
+        else:
+            paths_audio = filehandler.shift_get_files(
+                results, sample_rate=cor_sample_rate
+            )
+
+        new_results = self._align(
+            filename_list=None,
+            file_dir=None,
+            technique=technique,
+            filter_matches=filter_matches,
+            locality=locality,
+            locality_filter_prop=locality_filter_prop,
+            cor_sample_rate=cor_sample_rate,
+            max_lags=max_lags,
+            fine_aud_file_dict=paths_audio,
+            **kwargs,
+        )
+        if new_results is None:
+            print("No matches found for fine alignment")
+            return
+        new_results = align.combine_fine(results, new_results)
+
+        if destination_path:
+            copy_dict = {}
+            for name, value in new_results.items():
+                if name not in ["names_and_paths", "match_info", "fine_match_info"]:
+                    copy_dict[name] = value
+            try:
+                self._write_shifted_files(
+                    copy_dict,
+                    destination_path,
+                    new_results["names_and_paths"],
+                    write_extension,
+                )
+            except PermissionError:
+                print("Permission Denied for write fine_align")
+
+        return new_results
+
     @staticmethod
     def _write_shifted_files(
         files_shifts: dict,
@@ -1231,56 +1366,3 @@ class Audalign:
             num_processes=self.num_processors,
             **kwargs,
         )
-
-
-def _fingerprint_worker(
-    file_path: str,
-    hash_style="panako_mod",
-    start_end: tuple = None,
-    plot=False,
-    accuracy=2,
-    freq_threshold=200,
-) -> Tuple:
-    """
-    Runs the file through the fingerprinter and returns file_name and hashes
-
-    Args
-        file_path (str): file_path to be fingerprinted
-        hash_style (str): which hash style to use : ['base','panako_mod','panako', 'base_three']
-        start_end (tuple(float, float), optional): Silences before and after start and end. (0, -1) Silences last second, (5.4, 0) silences first 5.4 seconds
-        plot (bool): displays the plot of the peaks if true
-        accuracy (int): which accuracy level 1-4
-        freq_threshold (int): what the freq threshold is in specgram bins
-
-    Returns
-    -------
-        file_name (str, hashes : dict{str: [int]}): file_name and hash dictionary
-    """
-
-    Audalign._set_accuracy(accuracy)
-    Audalign._set_freq_threshold(freq_threshold)
-
-    file_name = os.path.basename(file_path)
-
-    try:
-        channel, _ = filehandler.read(file_path, start_end=start_end)
-    except FileNotFoundError:
-        print(f'"{file_path}" not found')
-        return None, None
-    except (
-        CouldntDecodeError,
-        IndexError,
-    ):  # Pydub throws IndexErrors for some files on Ubuntu (json, txt, others?)
-        print(f'File "{file_name}" could not be decoded')
-        return None, None
-
-    print(f"Fingerprinting {file_name}")
-    hashes = fingerprint.fingerprint(
-        channel,
-        hash_style=hash_style,
-        plot=plot,
-    )
-
-    print(f"Finished fingerprinting {file_name}")
-
-    return file_name, hashes
