@@ -34,6 +34,8 @@ def find_files(path, extensions=["*"]):
 
 
 def create_audiosegment(filepath: str, start_end: tuple = None, sample_rate=DEFAULT_FS):
+    if sample_rate is None:
+        sample_rate = DEFAULT_FS
     audiofile = AudioSegment.from_file(filepath)
     audiofile = audiofile.set_frame_rate(sample_rate)
     audiofile = audiofile.set_sample_width(2)
@@ -281,57 +283,105 @@ def _remove_noise(
         print(f"    Coudn't Decode {file_path}")
 
 
-def shift_write_files(files_shifts, destination_path, names_and_paths, write_extension):
+def shift_get_files(results: dict, sample_rate: int = None):
+    names_and_paths = results.pop("names_and_paths")
+    temp_a = results.pop("match_info")
 
+    shifts_files = _shift_files(
+        results,
+        None,
+        names_and_paths,
+        None,
+        sample_rate=sample_rate,
+        return_files=True,
+    )
+    results["names_and_paths"] = names_and_paths
+    results["match_info"] = temp_a
+    return shifts_files
+
+
+def shift_write_files(
+    files_shifts: dict,
+    destination_path: str,
+    names_and_paths: dict,
+    write_extension: str,
+):
+    _shift_files(
+        files_shifts,
+        destination_path,
+        names_and_paths,
+        write_extension,
+        return_files=False,
+    )
+
+
+def _shift_files(
+    files_shifts: dict,
+    destination_path: str,
+    names_and_paths: dict,
+    write_extension: str,
+    sample_rate: int = None,
+    return_files: bool = False,
+):
+    if sample_rate is None:
+        sample_rate = DEFAULT_FS
     max_shift = max(files_shifts.values())
 
     if write_extension:
         if write_extension[0] != ".":
             write_extension = "." + write_extension
 
-    audsegs = []
+    audsegs = {}
     for name in files_shifts.keys():
         file_path = names_and_paths[name]
 
         silence = AudioSegment.silent(
-            (max_shift - files_shifts[name]) * 1000, frame_rate=DEFAULT_FS
+            (max_shift - files_shifts[name]) * 1000, frame_rate=sample_rate
         )
 
-        audiofile = create_audiosegment(file_path)
+        audiofile = create_audiosegment(file_path, sample_rate=sample_rate)
 
         file_name = os.path.basename(file_path)
-        destination_name = os.path.join(destination_path, file_name)
-        audiofile = silence + audiofile
+        audiofile: AudioSegment = silence + audiofile
+        if not return_files:
+            destination_name = os.path.join(destination_path, file_name)
 
-        if os.path.splitext(destination_name)[1] in cant_write_ext:
-            destination_name = os.path.splitext(destination_name)[0] + ".wav"
+            if os.path.splitext(destination_name)[1] in cant_write_ext:
+                destination_name = os.path.splitext(destination_name)[0] + ".wav"
 
-        if write_extension:
-            destination_name = os.path.splitext(destination_name)[0] + write_extension
-
-            print(f"Writing {destination_name}")
-
-            with open(destination_name, "wb") as file_place:
-                audiofile.export(
-                    file_place, format=os.path.splitext(destination_name)[1][1:]
+            if write_extension:
+                destination_name = (
+                    os.path.splitext(destination_name)[0] + write_extension
                 )
 
-        else:
-            print(f"Writing {destination_name}")
+                print(f"Writing {destination_name}")
 
-            with open(destination_name, "wb") as file_place:
-                audiofile.export(
-                    file_place, format=os.path.splitext(destination_name)[1][1:]
-                )
+                with open(destination_name, "wb") as file_place:
+                    audiofile.export(
+                        file_place, format=os.path.splitext(destination_name)[1][1:]
+                    )
 
-        audsegs += [audiofile]
+            else:
+                print(f"Writing {destination_name}")
+
+                with open(destination_name, "wb") as file_place:
+                    audiofile.export(
+                        file_place, format=os.path.splitext(destination_name)[1][1:]
+                    )
+
+        audsegs[file_path] = audiofile
+
+    if return_files:
+        return audsegs
+
+    audsegs = list(audsegs.values())
 
     # adds silence to end of tracks to make them equally long for total
     longest_seconds = max(audseg.duration_seconds for audseg in audsegs)
     for i in range(len(audsegs)):
         audsegs[i] = audsegs[i] + AudioSegment.silent(
             (longest_seconds - audsegs[i].duration_seconds) * 1000,
-            frame_rate=DEFAULT_FS,
+            frame_rate=sample_rate,
         )
 
     # lower volume so the sum is the same volume
