@@ -10,6 +10,7 @@ def recognize(
     locality: float,
     locality_filter_prop: float,
     start_end: tuple,
+    max_lags: float = None,
 ):
     """
     Recognizes given file against already fingerprinted files
@@ -27,7 +28,7 @@ def recognize(
         None : if no match
     """
     if locality is not None:  # convert from seconds to samples
-        locality = max(
+        locality = max(  # turns into frames
             int(
                 locality
                 // (
@@ -38,6 +39,18 @@ def recognize(
             ),
             1,
         )
+    if max_lags is not None:
+        max_lags = max(  # turns into frames
+            int(
+                max_lags
+                // (
+                    fingerprint.DEFAULT_WINDOW_SIZE
+                    / fingerprint.DEFAULT_FS
+                    * fingerprint.DEFAULT_OVERLAP_RATIO
+                )
+            ),
+            1,
+        )
 
     t = time.time()
     matches = find_matches(audalign_object, file_path, start_end=start_end)
@@ -54,7 +67,12 @@ def recognize(
     file_match = None
     if len(rough_match) > 0:
         file_match = process_results(
-            audalign_object, rough_match, locality, filter_matches, filter_set
+            audalign_object,
+            rough_match,
+            locality,
+            filter_matches,
+            filter_set,
+            max_lags=max_lags,
         )
     t = time.time() - t
 
@@ -304,7 +322,12 @@ def find_loc_matches(matches_list: list, locality: int):
 
 
 def process_results(
-    audalign_object, results, locality, filter_matches=1, filter_set=False
+    audalign_object,
+    results,
+    locality,
+    filter_matches: int = 1,
+    filter_set: bool = False,
+    max_lags: float = None,
 ):
     """
     Takes matches from align_matches, filters and orders them, returns dictionary of match info
@@ -331,6 +354,13 @@ def process_results(
         match_offsets = sorted(match_offsets, reverse=True, key=lambda x: x[0][0])
         if match_offsets[0][0][0] <= filter_matches:
             continue
+        if max_lags is not None:  # TDDO Verify working gingerprints max_lags
+            i = 0
+            while i < len(match_offsets):
+                if abs(match_offsets[i][1]) > max_lags:
+                    match_offsets.pop(i)
+                    continue
+                i += 1
         for i in match_offsets:
             if i[0][0] <= filter_matches:
                 continue
